//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     nint.cxx
// Purpose:    Default Negation Interpretation (Belief Interpretation)
// Last Change:11.03.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2002 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "out.h"
#include "check.h"
#include "gneg.h"

#include "nint.h"

//=============================================================================
// Class Attributes (Static Private Class Members):
//=============================================================================

//-----------------------------------------------------------------------------
// NumCritNeg: Number of critical default negations.
//-----------------------------------------------------------------------------

int nint_c::NumCritNeg = -1;

//-----------------------------------------------------------------------------
// AllTrue: Maximal interpretation (all default negations are true).
//-----------------------------------------------------------------------------

nint_t nint_c::AllTrue = 0;

//=============================================================================
// Exported Class Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// init: Initialize this module.
//-----------------------------------------------------------------------------

void	nint_c::init(int num_crit_neg)
{
	int i;

	// Check Parameter:
	CHECK(num_crit_neg >= 0, "nint_c::init: num_crit_neg is negative");
	CHECK(num_crit_neg < VER_MAX_CRIT_NEG,
		"nint_c::init: num_crit_neg > VER_MAX_CRIT_NEG");
	CHECK(num_crit_neg<NINT_BITS && VER_MAX_CRIT_NEG<=NINT_BITS,
		"nint_c::init: num_crit_neg larger than NINT_BITS");

	// Store number of critical negations:
	NumCritNeg = num_crit_neg;

	// Compute maximal interpretation (all default negations are true):
	AllTrue = 0;
	for(i = 0; i < num_crit_neg; i++)
		AllTrue = (AllTrue << 1) + 1;
}

//-----------------------------------------------------------------------------
// print: Print an interpretation for the critical default negations.
//-----------------------------------------------------------------------------

void	nint_c::print(nint_t neg_int)
{
	int i;

	// Check initialization of this module:
	CHECK(NumCritNeg >= 0,
		"nint_c::print: module not initialized");
	CHECK(NumCritNeg <= NINT_BITS,
		"nint_c::print: NumCritNeg is invalid (1)");
	CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
		"nint_c::print: NumCritNeg is invalid (2)");

	// Check parameter:
	CHECK(neg_int <= AllTrue, "nint_c::print: neg_int is invalid");

	// Treat the interpretation for the empty set:
	if(NumCritNeg == 0) {
		out_c::empty_defneg_interpretation();
		return;
	}

	// Start unordered list (if HTML output):
	out_c::list_begin();

	// Loop over critical default negations:
	for(i = 0; i < NumCritNeg; i++) {
		out_c::list_item();
		gneg_c::get_crit_neg(i)->print();
		out_c::print_str(": ");
		out_c::print_bool(nint_c::is_true(neg_int, i));
		out_c::print_nl();
	}

	// End unordered list (if HTML output):
	out_c::list_end();
}

//-----------------------------------------------------------------------------
// get_curr_neg_int: Query current truth values of facts to determine interp.:
//-----------------------------------------------------------------------------

nint_t	nint_c::get_curr_neg_int(void)
{
	int i;
	gneg_t neg;
	gneg_l_t neg_elem;
	glit_t fact;
	bool_t neg_is_true;
	nint_t neg_int;

	// Check initialization of this module:
	CHECK(NumCritNeg >= 0,
		"nint_c::get_curr_neg_int: module not initialized");
	CHECK(NumCritNeg <= NINT_BITS,
		"nint_c::get_curr_neg_int: NumCritNeg is invalid (1)");
	CHECK(NumCritNeg <= VER_MAX_CRIT_NEG,
		"nint_c::get_curr_neg_int: NumCritNeg is invalid (2)");

	// Loop over critical default negations:
	neg_int = 0;
	for(i = 0; i < NumCritNeg; i++) {
		neg = gneg_c::get_crit_neg(i);
		neg_is_true = BOOL_FALSE;
		for(neg_elem = neg->get_elems(); neg_elem;
				neg_elem = neg_elem->get_next_neg_elem()) {
			fact = neg_elem->get_lit();
			CHECK(fact->is_false() || fact->is_true(),
			    "nint_c::get_curr_neg_int: undefined truth value");
			if(fact->is_false()) {
				neg_is_true = BOOL_TRUE;
				break;
			}
		}
		if(neg_is_true)
			neg_int = neg_int | (0x1 << i);
	}

	// Return current interpretation for default negation literals:
	return(neg_int);
}

